@@ -20,12 +20,14 @@
 #' @param max.age Maximum age of a set of equal length intervals. If no value is specified (max = NULL), the function uses a maximum age based on tree height.
 #' @param show.axis If TRUE plot x-axis (default = TRUE).
 #' @param binned If TRUE fossils are plotted at the mid point of each interval.
-#' @param show.proxy If TRUE add profile of sampling data to plot (e.g rates in time-dependent rates model) (default = FALSE).
+#' @param show.proxy If TRUE add profile of sampling data to plot (e.g. rates in time-dependent rates model) (default = FALSE).
 #' @param proxy.data Vector of sampling proxy data (default = NULL). Should be as long as the number of stratigraphic intervals.
 #' @param show.preferred.environ If TRUE add species preferred environmental value (e.g. water depth) (default = FALSE). Only works if combined with \code{show.proxy = TRUE}.
 #' @param preferred.environ Preferred environmental value (e.g. water depth). Currently only one value can be shown.
 #' @param show.taxonomy If TRUE highlight species taxonomy.
 #' @param taxonomy Taxonomy object.
+#' @param show.unknown If TRUE plot fossils with unknown taxonomic affiliation (i.e. sp = NA) (default = FALSE).
+#' @param reconstructed If TRUE plot the reconstructed tree. If fossils object contains no extant samples, the function assumes rho = 1 and includes all species at the present.
 #' @param root.edge If TRUE include the root edge (default = TRUE).
 #' @param hide.edge If TRUE hide the root edge but still incorporate it into the automatic timescale (default = FALSE).
 #' @param edge.width A numeric vector giving the width of the branches of the plotted phylogeny. These are taken to be in the same order as the component edge of \code{tree}. If fewer widths are given than the number of edges, then the values are recycled.
@@ -71,6 +73,7 @@
 #' @export
 #' @importFrom graphics par points lines text axis mtext segments rect plot
 #' @importFrom grDevices colors rgb adjustcolor
+#' @importFrom stats na.omit
 plot.fossils = function(x, tree, show.fossils = TRUE, show.tree = TRUE, show.ranges = FALSE,
                         # age info/options
                         show.strata = FALSE, strata = 1, max.age = NULL, interval.ages = NULL, binned = FALSE, show.axis = TRUE,
@@ -78,9 +81,9 @@ plot.fossils = function(x, tree, show.fossils = TRUE, show.tree = TRUE, show.ran
                         show.proxy = FALSE, proxy.data = NULL,
                         show.preferred.environ = FALSE, preferred.environ = NULL,
                         # taxonomy
-                        show.taxonomy = FALSE, taxonomy = NULL,
+                        show.taxonomy = FALSE, taxonomy = NULL, show.unknown = FALSE,
                         # tree appearance
-                        root.edge = TRUE, hide.edge = FALSE, edge.width = 1, show.tip.label = FALSE, align.tip.label = FALSE,
+                        root.edge = TRUE, hide.edge = FALSE, edge.width = 1, show.tip.label = FALSE, align.tip.label = FALSE, reconstructed = FALSE,
                         # fossil appearance
                         fossil.col = 1, range.col = rgb(0,0,1), extant.col = NULL, cex = 1.2, pch = 18, ...) {
 
@@ -109,7 +112,7 @@ plot.fossils = function(x, tree, show.fossils = TRUE, show.tree = TRUE, show.ran
   if(!"phylo" %in% class(tree))
     stop("tree must be an object of class \"phylo\"")
 
-  if(!all(fossils$edge %in% tree$edge))
+  if(!all( as.vector(na.omit(fossils$edge)) %in% tree$edge))
     stop("Mismatch between fossils and tree objects")
 
   # tolerance for extant tips and interval/ fossil age comparisons
@@ -125,6 +128,14 @@ plot.fossils = function(x, tree, show.fossils = TRUE, show.tree = TRUE, show.ran
   offset = 0 # distance from youngest tip to present
   if(!is.null(tree$origin.time)) offset = min(n.ages(tree))
 
+  # note max.age defined above based on the complete tree
+  if(reconstructed){
+    out = reconstructed.tree.fossils.objects(fossils, tree)
+    fossils = out$fossils
+    tree = out$tree
+    if(is.null(tree$root.edge)) root.edge = FALSE
+  }
+
   # check the tree
   Ntip <- length(tree$tip.label)
   if (Ntip < 2) {
@@ -217,7 +228,9 @@ plot.fossils = function(x, tree, show.fossils = TRUE, show.tree = TRUE, show.ran
     tmp = tmp + label.offset
   x.lim[2] = tmp
 
-  y.lim = c(1, Ntip)
+  if(show.unknown)
+    y.lim = c(0, Ntip)
+  else y.lim = c(1, Ntip)
 
   # define interval ages
   if(show.strata || show.axis || binned || show.proxy){
@@ -383,7 +396,7 @@ plot.fossils = function(x, tree, show.fossils = TRUE, show.tree = TRUE, show.ran
         fossils$col[which(fossils$h < tol)] = extant.col
       }
 
-      if(show.fossils || show.ranges) {
+      if(show.fossils || show.ranges){
         if(binned) {
           fossils$r = sapply(fossils$h - offset, function(x) {
             if(x < tol) return(max(xx) - x)
@@ -394,6 +407,14 @@ plot.fossils = function(x, tree, show.fossils = TRUE, show.tree = TRUE, show.ran
         }
       }
 
+      # fossils unaffiliated with taxonomy
+      if( show.unknown && any(is.na(fossils$sp)) ){
+        unknownx = fossils[ which(is.na(fossils$sp)), ]$r
+        unknowny = rep(0, length(unknownx))
+        points(unknownx, unknowny, cex = cex, pch = pch, col = fossil.col)
+        fossils = na.omit(fossils)
+      }
+
       # fossils
       if(show.fossils & !show.ranges){
         points(fossils$r, yy[fossils$edge], cex = cex, pch = pch, col = fossils$col)
@@ -512,9 +533,9 @@ add.depth.profile = function(depth.profile, axis.strata, strata, show.axis, add.
   par(fig = c(0,1,0,0.4), new = T)
   # change the y-axis scale for depth
   u = par("usr") # current scale
-  depth = rev(depth.profile)
-  tol = max(depth, PD) * 0.1
-  par(usr = c(u[1], u[2], min(depth, PD) - tol, max(depth, PD) + tol))
+  depth.profile = rev(depth.profile)
+  tol = max(depth.profile, PD) * 0.1
+  par(usr = c(u[1], u[2], min(depth.profile, PD) - tol, max(depth.profile, PD) + tol))
   time = c()
   depth = c()
   for(i in 1:length(depth.profile)){
